/* This file is part of the KDE project

   Copyright (C) 2004 Gary Cramblitt <garycramblitt@comcast.net>

   Adapted from kdeutils/ark/konqplugin by
        Georg Robbers <Georg.Robbers@urz.uni-hd.de>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; version 2
   of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include "akregatorplugin.h"

#include <kapplication.h>
#include <kmimetype.h>
#include <kdebug.h>
#include <kaction.h>
#include <kinstance.h>
#include <klocale.h>
#include <konq_popupmenu.h>
#include <kpopupmenu.h>
#include <kgenericfactory.h>
#include <kurl.h>
#include <dcopclient.h>
#include <kprocess.h>

#include <qdir.h>
#include <qcstring.h>
#include <qobject.h>
#include <qmessagebox.h>

typedef KGenericFactory<AkregatorMenu, KonqPopupMenu> AkregatorMenuFactory;
K_EXPORT_COMPONENT_FACTORY( libakregatorkonqplugin, AkregatorMenuFactory("akregatorkonqplugin") );

AkregatorMenu::AkregatorMenu( KonqPopupMenu * popupmenu, const char *name, const QStringList& /* list */ )
                : KonqPopupMenuPlugin( popupmenu, name)
{
    if ( QCString( kapp->name() ) == "kdesktop" && !kapp->authorize("editable_desktop_icons" ) )
        return;

    // Do nothing if user has turned us off.
    // TODO: Not yet implemented in aKregator settings.
    m_conf = new KConfig( "akregatorrc" );
    m_conf->setGroup( "AkregatorKonqPlugin" );
    if ( !m_conf->readBoolEntry( "Enable", true ) )
        return;

    KGlobal::locale()->insertCatalogue("akregatorkonqplugin");
    // Build list of supported MIME types in m_feedMimeTypes.
    feedMimeTypes();
    // Get the list of URLs clicked on from Konqi.
    m_list = popupmenu->fileItemList();
    // See if any are RSS feeds.
    KFileItemListIterator it( m_list );
    KFileItem * item;
    bool hasFeed = false;
    while ( ( item = it.current() ) != 0 )
    {
        ++it;
        if (isFeedUrl(item))
        {
            kdDebug() << "Akregator::Menu found feed URL " << item->url().prettyURL() << endl;
            hasFeed = true;
            break;
        }
    }

    // If any of the URLs are RSS feeds, add entry to Konqi popup menu.
    if (hasFeed)
    {
        KAction * action;
        action = new KAction( i18n( "Add Feed to aKregator" ), "akregator", 0, this,
            SLOT( slotAddFeed() ), actionCollection(), "akregatorkonqplugin_mnu" );
        addAction( action );
        addSeparator();
    }
}

AkregatorMenu::~AkregatorMenu()
{
}

void AkregatorMenu::feedMimeTypes()
{
    m_feedMimeTypes << "text/rss";
    m_feedMimeTypes << "text/rdf";
    m_feedMimeTypes << "text/xml";
}

bool AkregatorMenu::isFeedUrl(const KFileItem * item)
{
    if ( m_feedMimeTypes.contains( item->mimetype() ) )
        return true;
    else
    {
        QString url = item->url().prettyURL();
        // If URL ends in .htm or .html, it is not a feed url.
        if (url.contains(".htm", false)) return false;
        if (url.contains("rss", false) != 0) return true;
        if (url.contains("rdf", false) != 0) return true;
        if (url.contains("xml", false) != 0) return true;
    }
    return false;
}

void AkregatorMenu::slotAddFeed()
{
    // Get the DCOP ID of aKregator application.  Any one will do, take first one found.
    DCOPClient *client = kapp->dcopClient();
    QCStringList dcopClients = client->registeredApplications();
    QCStringList::iterator clientIt;
    QCString akregatorClient;
    for (clientIt = dcopClients.begin(); (clientIt != dcopClients.end()) ; ++clientIt )
    {
        if ((*clientIt).contains("akregator", false) != 0) // FIXME: what happens when there's application named akregatorsomething?
        {
            akregatorClient = *clientIt;
            kdDebug() << "AkregatorMenu::slotAddFeed aKregator DCOP ID is " << akregatorClient << endl;
            break;
        }
    }

    if (akregatorClient.isNull())
    {
        // If aKregator not running, start it using KProcess and pass URLs via command line.
        KProcess *proc = new KProcess;
        *proc << "akregator" << "-g" << "Imported";
        KFileItemListIterator it( m_list );
        KFileItem * item;
        while (  ( item = it.current() ) != 0 )
        {
            ++it;
            if (isFeedUrl(item)) *proc << "-a" << item->url().prettyURL();
        }
        proc->start(KProcess::DontCare);
        delete proc;
    }
    else
    {
        // Locate the object whose name begins with aKregatorPart.  Any one will do, take the first found.
        QCStringList akregatorObjs = client->remoteObjects(akregatorClient);
        QCString akregatorObj;
        for (clientIt = akregatorObjs.begin(); (clientIt != akregatorObjs.end()) ; ++clientIt )
        {
            if ((*clientIt).contains("aKregatorIface", false) != 0)
            {
                akregatorObj = *clientIt;
                kdDebug() << "AkregatorMenu::slotAddFeed aKregator object is " << akregatorObj << endl;
                break;
            }
        }
        if (akregatorObj.isNull()) 
            return;
       
        KFileItemListIterator it( m_list );
        KFileItem * item;
        while (  ( item = it.current() ) != 0 )
        {
            ++it;
            if (isFeedUrl(item))
            {
                kdDebug() << "AkregatorMenu::slotAddFeed Telling aKregator to add feed " << item->url().prettyURL() << endl;
                QByteArray  data;
                QCString    replyType;
                QByteArray  replyData;
                QDataStream arg(data, IO_WriteOnly);
                arg << item->url().prettyURL();
                arg << i18n("Imported");
                if ( !client->call(akregatorClient, akregatorObj, "addFeedToGroup(QString,QString)",
                    data, replyType, replyData, true) )
                QMessageBox::warning( 0, i18n( "DCOP Call failed" ),
                    i18n( "The DCOP call addFeedToGroup failed" ));
            }
        }
    }
}

#include "akregatorplugin.moc"
